#! /usr/bin/python

# Copyright (c) 2015 Cumulus Networks, Inc.  All rights reserved
# Author: David Yen <dhyen@cumulusnetworks.com>

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import eeprom_base
    import eeprom_tlvinfo
    import eeprom_ipmi_fru
except ImportError, e:
    raise ImportError(str(e) + "- required module not found")

#
# (cpu) board
#
class board(eeprom_tlvinfo.TlvInfoDecoder):
    EEPROM_MAX_SIZE = 0x2000

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path,
                                    0,
                                    "",
                                    ro,
                                    self.EEPROM_MAX_SIZE)

#
# power supply unit
#
class psu_dc_format(eeprom_base.eepromDecoder):
    ps_fmt = [('burn', 'x', 12),
              ('manufacturer', 's', 5),
              ('burn', 'x', 1),
              ('part_num', 's', 11),
              ('burn', 'x', 1),
              ('model_num', 's', 10),
              ('burn', 'x', 1),
              ('revision', 's', 3),
              ('burn', 'x', 1),
              ('serial_num', 's', 14)]

    supported_models = ['CLS-800K', ]

    def __init__(self, name, path, cpld_root, ro):
        cpld_root = '/sys/class/gpio'
        self.cpld_path = cpld_root + '/psu_pwr' +  \
            name.split('psu')[1] + '_all_ok/value'
        super(psu_dc_format, self).__init__(path,
                                            self.ps_fmt,
                                            0,
                                            self.cpld_path,
                                            ro)
        self.e = self.read_eeprom()

    def decoder(self, s):
        # not implemented
        return 0

    def check_presence(self):
        path = self.cpld_path
        try:
            fp = open(path, 'r')
            status = fp.readline().rstrip()
            if int(status):
                return 1
            else:
                return 0
        except IOError:
            return 0

    def check_status(self):
        status = super(psu_dc_format, self).check_status()
        model_num = self.model_num()
        if self.check_presence():
            if model_num in self.supported_models:
                return 'ok'
            else:
                return 'ac psu'
        else:
            return status

    def model_num(self):
        loc = 0
        e = self.e
        for I in self.f:
            end = loc + I[2]
            t = e[loc:end].replace('\0', '.')
            loc = end
            if I[0] == 'part_num':
                return t[:8]
        return 'bad_model_num'


    def is_checksum_valid(self, e):
        # not implemented
        return (True, 0)

class psu_ac_fru_format(eeprom_ipmi_fru.IpmiFruDecoder):
    def check_status(self):
        cpld_root = '/sys/class/gpio'
        cpld_path = cpld_root + '/psu_pwr' +  \
            self.name.split('psu')[1] + '_present/value'
        try:
            with open(cpld_path, "r") as fp:
                status = fp.readline().rstrip()
            if int(status):
                return 'ok'
            return 'absent'
        except IOError:
            return 'absent'

def psu(name, path, cpld_root, ro):
    #
    # First try DC PSU format.  If it doesn't look like that,
    # try IPMI FRU format.
    #
    psu_dc = psu_dc_format(name, path, cpld_root, ro)
    stat = psu_dc.check_status()
    if stat == 'ok':
        return psu_dc

    psu_ac_fru = psu_ac_fru_format(name, path, cpld_root, ro)
    stat = psu_ac_fru.check_status()
    if stat == 'ok':
        return psu_ac_fru
    return stat

#
# fans
#
class fan(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x100
    EEPROM_OFFSET = 0x0

    def __init__(self, name, path, cpld_root, ro):
        self.name = name
        super(fan, self).__init__(path,
                                  self.EEPROM_OFFSET,
                                  "",
                                  ro,
                                  self.EEPROM_MAX_SIZE)

    def check_status(self):
        path = '/sys/class/gpio/fan' \
               + self.name.split('fan')[1] \
               + '_present/value'

        try:
            with open(path, "r") as fp:
                status = fp.readline().rstrip()
                if status == '1':
                    return 'ok'
                return 'absent'
        except IOError:
            return 'absent'
