#! /usr/bin/python
# Copyright 2012 Cumulus Networks LLC, all rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import binascii
    import time
    import optparse
    import warnings
    import os
    import sys
    import subprocess
    import tempfile
    import eeprom_base
    import eeprom_tlvinfo
    import eeprom_ipmi_fru

except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

# The Celestica Smallstone has two EEPROM formats: an older fixed field
# format (modified NXID format) and the current TLV based format.

# Current TLV based EEPROM format
class board_tlv_format(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x2000

    def __init__(self, name, path, cpld_root, ro):
        super(board_tlv_format, self).__init__(path, 0, "", ro, self.EEPROM_MAX_SIZE)

def bcd_d2b(d):
    '''
    Binary Code Decimal

    Convert decimal number to binary coded decimal

    d - A decimal integer

    Returns an integer
    '''

    # Only expect two digit numbers
    if d > 99:
        raise RuntimeError

    return ((d / 10) * 16) + (d % 10)

# Celestica NXID
class celestica_nxid_format(eeprom_base.eepromDecoder):
    brd_fmt = [ ('id', 's', 4),
                ('prod_name', 's', 12),
                ('errata_level', 's', 5),
                ('build_date', 'C', 6),
                ('burn', 'x', 1),
                ('version', 'C', 4),
                ('burn', 'x', 8),
                ('burn', 'x', 2),
                ('burn', 'x', 1),
                ('serial_num', 's', 21),
                ('mac_count', 'C', 1),
                ('mac_flag', 's', 1),
                ('burn', 'x', 6),
                ('macaddr', 'C', 6),
                ('burn', 'x', 36),
                ('crc', 'C', 4)
              ]

    def __init__(self, name, path, cpld_root, ro):
        eeprom_base.eepromDecoder.__init__(self,path, self.brd_fmt, 0, '', ro)

    def base_mac_addr(self, e):
        loc = 0
        for I in self.brd_fmt:
            end = loc + I[2]
            t = e[loc:end]
            loc = end
            if I[0] != 'macaddr':
                continue
            else:
                mac = ":".join([binascii.b2a_hex(T) for T in list(t)])
                return mac
        raise RuntimeError("did not find mac address")

    def switchaddrrange(self, e):
        # using 129 => (1 + 32 * 4)
        return 129

    def decoder(self, s, t):
        if s == 'version':
            i = ".".join([str(ord(T)) for T in list(t)])
        elif s == 'build_date':
            # Raw date data is in binary coded decimal
            i = "20" + \
                '%02x' % ord(t[0]) + "/" + \
                '%02x' % ord(t[1]) + "/" + \
                '%02x' % ord(t[2]) + " " + \
                '%02x' % ord(t[3]) + ":" + \
                '%02x' % ord(t[4]) + ":" + \
                '%02x' % ord(t[5])
        elif s == 'mac_count':
            i = str(ord(t[0]))
        elif s == 'macaddr':
            i = ":".join([binascii.b2a_hex(T) for T in list(t)]).upper()
        elif s == 'crc':
            i = "0x" + binascii.b2a_hex(t).upper()

        return i

    def encoder(self, I, v):
        l = ''
        try:
            if I[0] == 'id':
                l += 'NXID'
            elif I[0] == 'version':
                errstr = 'x.x.x.x where x is a digit'
                if len(v.split('.')) != 4:
                    raise ValueError
                l += ''.join(chr(int(C)) for C in v.split('.'))
            elif I[0] == 'build_date':
                errstr = 'YYYY/MM/DD hh:mm:ss'
                [f1, j, f2] = v.rstrip('\n\r').partition(' ')
                [y, m ,d] = f1.split('/')
                [hh, mm, ss] = f2.strip().split(':')
                if int(m) not in range(1,13) or int(d) not in range(1,32) or \
                   int(y) < 2000 or int(hh) > 23 or int(mm) > 59 or int(ss) > 59:
                    raise
                l += chr(bcd_d2b(int(y) - 2000)) + chr(bcd_d2b(int(m))) + chr(bcd_d2b(int(d)))
                l += chr(bcd_d2b(int(hh))) + chr(bcd_d2b(int(mm))) + chr(bcd_d2b(int(ss)))
            elif I[0] == 'mac_count':
                errstr = 'numeric'
                l += chr(int(v))
            elif I[0] == 'macaddr':
                errstr = 'xx:xx:xx:xx:xx:xx  where xx is alpha-numeric'
                if len(v.split(':')) != 6:
                    raise
                mac = v.replace(':', '', 5)
                l += binascii.a2b_hex(mac)
            elif I[1] == 's':
                # left adjust the field to the defined length
                l += v.ljust(I[2])[:I[2]]
        except:
            sys.stderr.write("Error: '" + I[0] + "' correct format is " + errstr + "\n")
            exit(0)

        return l

    def is_checksum_field(self, I):
        return I[0] == 'crc'

    def checksum_field_size(self):
        return 4

    def checksum_type(self):
        return 'crc32'

    def serial_number_str(self, e):
        return self.find_field(e, 'serial_num')

def board(name, path, cpld_root, ro):
    '''
    Look to see which type of board EEPROM format we have.  Based
    on that we return an instance of the appropriate class.

    Check for a valid TLV header.  If found assume we have the TLV based
    format.  Otherwise assume it is the older format.
    '''

    # Try new format first
    new_format = board_tlv_format(name, path, cpld_root, ro)
    F = new_format.open_eeprom()
    h = F.read(new_format._TLV_INFO_HDR_LEN)
    if len(h) != new_format._TLV_INFO_HDR_LEN:
        raise RuntimeError("expected to read %d bytes from %s, " % \
                           (new_format._TLV_INFO_HDR_LEN, path) + \
                           "but only read %d" % (len(h)))
    F.close()
    if new_format.is_valid_tlvinfo_header(h):
        return new_format

    # Try old format
    old_format = celestica_nxid_format(name, path, cpld_root, ro)
    e = old_format.read_eeprom()
    (is_valid, valid_crc) = old_format.is_checksum_valid(e)
    if is_valid:
        return old_format

    # Unknown format or corrupt.  Fallback to the new format so that
    # writing will use the new format.
    return new_format

#
# power supply unit
#
class psu(eeprom_ipmi_fru.IpmiFruDecoder):

    def check_status(self):
        path = '/sys/devices/ffe05000.localbus/ffb00000.CPLD1/psu_pwr' + self.name.split('psu')[1] + \
            '_all_ok'
        try:
            with open(path, "r") as fp:
                status = fp.readline().rstrip()
            if int(status):
                return 'ok'
            return 'absent'
        except:
            return 'absent'
