#! /usr/bin/python
# Copyright 2012 Cumulus Networks LLC, all rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import binascii
    import time
    import optparse
    import warnings
    import os
    import sys
    import eeprom_base
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

#
# base board
#
class board(eeprom_base.eepromDecoder):
    brd_fmt = [ ('vendor_id', 's', 32),
                ('country_code', 's', 32),
                ('date_code', 's', 32),
                ('serial_number', 's', 32),
                ('part_number', 's', 32),
                ('revision', 's', 32),
                ('order_number', 's', 32),
                ('component_type', 's', 32),
                ('component_id', 's', 32),
                ('start_mac', 'C', 6),
                ('end_mac', 'C', 6),
                ('burn', 'x', 20),
                ('checksum', 'C', 1),
              ]

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path,             \
                                    self.brd_fmt,     \
                                    0,                \
                                    '',               \
                                    ro)

    def findmacfield(self, e, field):
        loc = 0
        mac = None 
        for I in self.brd_fmt:
            end = loc + I[2]
            t = e[loc:end]
            loc = end
            if I[0] != field:
                continue
            else:
                mac = ":".join([binascii.b2a_hex(T) for T in list(t)])
                return mac

    def base_mac_addr(self, e):
        (is_valid, valid_crc) = self.is_checksum_valid(e)
        if is_valid is False:
            return super(board, self).base_mac_addr(e)
        mac = self.findmacfield(e, 'start_mac')
        if mac is not None :
            return mac
        else:
            return super(board, self).base_mac_addr(e)

    def switchaddrrange(self, e):
        (is_valid, valid_crc) = self.is_checksum_valid(e)
        if is_valid is False:
            return 3 # assume there are 3 mac addresses allocated when we don't know
        mac1 = self.findmacfield(e, 'start_mac')
        mac2 = self.findmacfield(e, 'end_mac')
        if mac1 is not None and mac2 is not None :
            return self.diff_mac(mac1, mac2)
        else:
            return 3

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        elif s == 'start_mac' or s == 'end_mac':
            i = ":".join([binascii.b2a_hex(T) for T in list(t)])
        else:
            i = t
        return i

    def encoder(self, I, v):
        l = ''
        try:
            if I[1] == 's':
                # left adjust the field to the defined length
                l += v.ljust(I[2], '\x00')[:I[2]]
            if I[0] == 'start_mac' or I[0] == 'end_mac':
                errstr = 'xx:xx:xx:xx:xx:xx  where xx is alpha-numeric'
                if len(v.split(':')) != 6:
                    raise
                mac = v.replace(':', '', 5)
                l += binascii.a2b_hex(mac)
        except:
            sys.stderr.write("Error: '" + I[0] + "' correct format is " + errstr + "\n")
            exit(0)

        return l

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 1

    def checksum_type(self):
        return '2s-complement'


#
# power supply unit
#
class psu(eeprom_base.eepromDecoder):

    ps_fmt = [ ('order_number', 's', 32),
               ('description', 's', 32),
               ('serial_num', 's', 16),
               ('mfg_pn', 's', 16),
               ('customer_pn', 's', 16),
               ('mfg_date', 's', 16),
               ('mfg_location', 's', 16),
               ('RoHs compliance', 's', 16),
               ('mfg_deviations', 's', 32),
               ('odm_pn', 's', 32),
               ('fan_air_flow_id', 'c', 1),
               ('checksum', 'c', 1)
             ]

    def __init__(self, name, path, cpld_root, ro):
        cpldpath = cpld_root + '/psu_pwr' + name.split('psu')[1]
        super(psu, self).__init__(path,         \
                                  self.ps_fmt,  \
                                  0,            \
                                  cpldpath,     \
                                  ro)

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        status = super(psu, self).check_status()

        # each device may have a different status string, decide what 
        # translates to 'ok' here.
        if status.startswith('installed'):
            return 'ok'
        else:
            return status

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 1

    def checksum_type(self):
        return '2s-complement'
