#!/bin/sh

#
# Copyright 2015,2016,2018 Cumulus Networks, Inc.  All rights reserved
# All rights reserved.
#

# Invoked by /etc/init.d/hw_init

# Determine the fan configuration for multiple Dell S4000 SKUs and
# setup the fan related configuration files accordingly.

# The fan configuration files need only be setup once after a fresh
# install.

# Cache previously detected fan configuration
CACHE_DIR=/run/cache/cumulus/fan_config
CACHE_CONF="$CACHE_DIR/state"

SENSORS_CONF=/etc/sensors.d/dell_s4000_c2338.conf

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

detect_fans()
{
    # f2b_part_nums = ['0J09D3', '0KMJ74', '0M68YC']
    # b2f_part_nums = ['04NM4C', '0RM2JW', '099TJK']
    part_num=$(decode-syseeprom |grep 'Part Number'|awk '{print $5}')
    case "$part_num" in
    0J09D3|0KMJ74|0M68YC)
        # F2B part nums
        FAN_CONFIG="F2B"
        ;;
    04NM4C|0RM2JW|099TJK)
        # B2F part nums
        FAN_CONFIG="B2F"
        ;;
    *)
        # Default to B2F
        FAN_CONFIG="B2F"
        ;;
    esac
    # Store result in the cache
    mkdir -p $CACHE_DIR || {
        log_failure_msg "Problems making cache dir: $CACHE_DIR"
    }
    echo $FAN_CONFIG > $CACHE_CONF || {
        log_failure_msg "Problems writing cache file: $CACHE_CONF"
    }
}

# make_sensors_conf() -- Based on the fan type generate an appropriate
# sensors configuration file.
make_sensors_conf()
{
    config=$1

    # Emit common sensors config file text
    mkdir -p $(dirname $SENSORS_CONF)

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S4000 C2338 F2B
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
# On Dell S4000, U54, U245 and U244 sensors control thermal hardware shutdown.
# For these sesnors, when the temperature hits the temp1_max value, the box will
# undergo thermal hardware shutdown. The box will power up  once the temperature
# lowers to temp1_max_hyst.
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"

# U54: tmp75-i2c-11-4c: Board sensor near Networking ASIC.
chip "tmp75-i2c-11-4c"
     set temp1_max 77
     set temp1_max_hyst 67

# U245: tmp75-i2c-11-4e: Board sensor near QSFP+ cage.
chip "tmp75-i2c-11-4e"
     set temp1_max 60
     set temp1_max_hyst 50

# U244: tmp75-i2c-11-4d: Board sensor near SFP+ cage.
chip "tmp75-i2c-11-4d"
     set temp1_max 71
     set temp1_max_hyst 61

# U57: tmp75-i2c-10-4d: Board sensor near CPU
chip "tmp75-i2c-10-4d"
     set temp1_max 54
     set temp1_max_hyst 54

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-*"
     set temp1_max 55
     ignore temp2

# Fan sensors
chip "max6620-i2c-*-2a"
     ignore fan3
     ignore fan4

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# Hot swap controller
chip "ltc4215-*"
     ignore in1

EOF
    else
        # Use back-to-front configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S4000 C2338 B2F
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
# On Dell S4000, U54, U245 and U244 sensors control thermal hardware shutdown.
# For these sesnors, when the temperature hits the temp1_max value, the box will
# undergo thermal hardware shutdown. The box will power up once the temperature
# lowers to temp1_max_hyst.
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"

# U54: tmp75-i2c-11-4c: Board sensor near Networking ASIC.
chip "tmp75-i2c-11-4c"
     set temp1_max 70
     set temp1_max_hyst 60

# U245: tmp75-i2c-11-4e: Board sensor near QSFP+ cage.
chip "tmp75-i2c-11-4e"
     set temp1_max 60
     set temp1_max_hyst 50

# U244: tmp75-i2c-11-4d: Board sensor near SFP+ cage.
chip "tmp75-i2c-11-4d"
     set temp1_max 69
     set temp1_max_hyst 59

# U57: tmp75-i2c-10-4d: Board sensor near CPU
chip "tmp75-i2c-10-4d"
     set temp1_max 49
     set temp1_max_hyst 49

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-*"
     set temp1_max 55
     ignore temp2

# Fan sensors
chip "max6620-i2c-*-2a"
     ignore fan3
     ignore fan4

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# Hot swap controller
chip "ltc4215-*"
     ignore in1

EOF
    fi
}

# Main program starts here

if [ -r $CACHE_CONF ] ; then
    FAN_CONFIG=$(cat $CACHE_CONF)
    if [ $FAN_CONFIG != "F2B" ] && [ $FAN_CONFIG != "B2F" ] ; then
        log_failure_msg "Unexpected fan config: $FAN_CONFIG"
        exit 1
    fi
else
    detect_fans
    make_sensors_conf $FAN_CONFIG
fi
