#!/bin/sh

#
# Copyright 2015,2016,2018 Cumulus Networks, Inc.  All rights reserved
# All rights reserved.
#

# Invoked by /etc/init.d/hw_init

# Determine the fan configuration for multiple Dell S4048T SKUs and
# setup the fan related configuration files accordingly.

# The fan configuration files need only be setup once after a fresh
# install.

# Cache previously detected fan configuration
CACHE_DIR=/run/cache/cumulus/fan_config
CACHE_CONF="$CACHE_DIR/state"

SENSORS_CONF=/etc/sensors.d/dell_s4048t_c2338.conf

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

detect_fans()
{
    # f2b_part_nums = ['0YVCK0', '0DC9DH', '08D1MG']
    # b2f_part_nums = ['0J1P3P', '05JM5X', '0Y1K3R']
    part_num=$(decode-syseeprom |grep 'Part Number'|awk '{print $5}')
    case "$part_num" in
    0YVCK0|0DC9DH|08D1MG)
        # F2B part nums
        FAN_CONFIG="F2B"
        ;;
    0J1P3P|05JM5X|0Y1K3R)
        # B2F part nums
        FAN_CONFIG="B2F"
        ;;
    *)
        # Default to B2F
        FAN_CONFIG="B2F"
        ;;
    esac
    # Store result in the cache
    mkdir -p $CACHE_DIR || {
        log_failure_msg "Problems making cache dir: $CACHE_DIR"
    }
    echo $FAN_CONFIG > $CACHE_CONF || {
        log_failure_msg "Problems writing cache file: $CACHE_CONF"
    }
}

# make_sensors_conf() -- Based on the fan type generate an appropriate
# sensors configuration file.
make_sensors_conf()
{
    config=$1

    # Emit common sensors config file text
    mkdir -p $(dirname $SENSORS_CONF)

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S4048T C2338 F2B
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
# On Dell S4048T, U278, U245, U244, U254, U288, U277 and U57 sensors control thermal hardware shutdown.
# For these sesnors, when the temperature hits the temp1_max value, the box will
# undergo thermal hardware shutdown. The box will power up  once the temperature
# lowers to temp1_max_hyst.
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"

#U278: tmp75-i2c-11-4b: Temp Sensor close to Networking ASIC
chip "tmp75-i2c-11-4b"
     set temp1_max 67
     set temp1_max_hyst 57

#U245: tmp75-i2c-11-4e: Board Sensor near Left PHY 
chip "tmp75-i2c-11-4e"
     set temp1_max 72
     set temp1_max_hyst 62

#U244: tmp75-i2c-11-4a: Board Sensor near Right PHY 
chip "tmp75-i2c-11-4a"
     set temp1_max 72
     set temp1_max_hyst 62

#U254: tmp75-i2c-11-4c: Board Sensor near Middle PHY 
chip "tmp75-i2c-11-4c"
     set temp1_max 72
     set temp1_max_hyst 62

#U288: tmp75-i2c-11-49: Board Sensor near QSFP+
chip "tmp75-i2c-11-49"
     set temp1_max 62
     set temp1_max_hyst 52

#U277: tmp75-i2c-11-4f: Board Sensor near QSFP+
chip "tmp75-i2c-11-4f"
     set temp1_max 73
     set temp1_max_hyst 63

#U57: tmp75-i2c-10-4d: Board Sensor near CPU
chip "tmp75-i2c-10-4d"
     set temp1_max 65
     set temp1_max_hyst 55

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-*"
     set temp1_max 55
     ignore temp2

# Fan sensors
chip "emc2305-*"
     ignore fan5

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# Hot swap controller
chip "ltc4215-*"
     ignore in1

EOF
    else
        # Use back-to-front configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S4048T C2338 B2F
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
# On Dell S4048T, U278, U245, U244, U254, U288, U277 and U57 sensors control thermal hardware shutdown.
# For these sesnors, when the temperature hits the temp1_max value, the box will
# undergo thermal hardware shutdown. The box will power up  once the temperature
# lowers to temp1_max_hyst.
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"

#U278: tmp75-i2c-11-4b: Temp Sensor close to Networking ASIC
chip "tmp75-i2c-11-4b"
     set temp1_max 70
     set temp1_max_hyst 60

#U245: tmp75-i2c-11-4e: Board Sensor near Left PHY 
chip "tmp75-i2c-11-4e"
     set temp1_max 77
     set temp1_max_hyst 67

#U244: tmp75-i2c-11-4a: Board Sensor near Right PHY 
chip "tmp75-i2c-11-4a"
     set temp1_max 77
     set temp1_max_hyst 67

#U254: tmp75-i2c-11-4c: Board Sensor near Middle PHY 
chip "tmp75-i2c-11-4c"
     set temp1_max 77
     set temp1_max_hyst 67

#U288: tmp75-i2c-11-49: Board Sensor near QSFP+
chip "tmp75-i2c-11-49"
     set temp1_max 62
     set temp1_max_hyst 52

#U277: tmp75-i2c-11-4f: Board Sensor near QSFP+
chip "tmp75-i2c-11-4f"
     set temp1_max 67
     set temp1_max_hyst 57

#U57: tmp75-i2c-10-4d: Board Sensor near CPU
chip "tmp75-i2c-10-4d"
     set temp1_max 65
     set temp1_max_hyst 55

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-*"
     set temp1_max 55
     ignore temp2

# Fan sensors
chip "emc2305-*"
     ignore fan5

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# Hot swap controller
chip "ltc4215-*"
     ignore in1

EOF
    fi
}

# Main program starts here

if [ -r $CACHE_CONF ] ; then
    FAN_CONFIG=$(cat $CACHE_CONF)
    if [ $FAN_CONFIG != "F2B" ] && [ $FAN_CONFIG != "B2F" ] ; then
        log_failure_msg "Unexpected fan config: $FAN_CONFIG"
        exit 1
    fi
else
    detect_fans
    make_sensors_conf $FAN_CONFIG
fi
