#! /usr/bin/python
# Copyright 2014 Cumulus Networks, Inc.
# All rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import binascii
    import time
    import optparse
    import warnings
    import os
    import sys
    import subprocess
    import tempfile
    import struct
    import eeprom_base
    import eeprom_tlvinfo
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

#
# base board
#
class board(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x100
    EEPROM_OFFSET = 0x0

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path,
                                    self.EEPROM_OFFSET,
                                    "",
                                    ro,
                                    self.EEPROM_MAX_SIZE)


class psu(eeprom_base.eepromDecoder):

    ps_fmt = [ ('burn', 'h', 6),
               ('country_code', 's', 2),
               ('dell_part_num', 's', 6),
               ('mfg_id', 's', 5),
               ('date_code', 's', 3),
               ('serial_number', 's', 4),
               ('dell_part_num_rev', 's', 3),
               ('service_tag', 's', 7),
               ('part_num', 's', 10),
               ('part_num_rev', 's', 3),
               ('mfg_test_result', 's', 2),
               ('burn', 's', 11),
               ('checksum', 'c', 2)
               ]

    def __init__(self, name, path, cpld_root, ro):
        cpld_root = '/sys/devices/platform/dell_s6000_cpld.0'
        cpld_path = cpld_root + '/psu_pwr' + name.split('psu')[1]
        super(psu, self).__init__(path,         \
                                  self.ps_fmt,  \
                                  0,            \
                                  cpld_path,    \
                                  ro)

    def decoder(self, s, t):
        if s == 'checksum':
            t = binascii.b2a_hex(t).upper()
            t = struct.unpack(">H", struct.pack("<H", int(t,16)))[0]
            i = hex(t)
        elif s == 'header':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        status = super(psu, self).check_status()

        # each device may have a different status string, decide what
        # translates to 'ok' here.
        if status.startswith('installed, ok'):
            return 'ok'
        else:
            return 'not present'

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 2

    def checksum_type(self):
        return 'dell-crc'

class fan(eeprom_base.eepromDecoder):

    fan_fmt = [ ('burn', 'h', 6),
               ('country_code', 's', 2),
               ('dell_part_num', 's', 6),
               ('mfg_id', 's', 5),
               ('date_code', 's', 3),
               ('serial_number', 's', 4),
               ('dell_part_num_rev', 's', 3),
               ('service_tag', 's', 7),
               ('part_num', 's', 10),
               ('part_num_rev', 's', 3),
               ('mfg_test_result', 's', 2),
               ('burn', 's', 11),
               ('checksum', 'c', 2)
               ]

    def __init__(self, name, path, cpld_root, ro):
        cpld_root = '/sys/devices/platform/dell_s6000_cpld.0'
        x = int(name.split('fan')[1]) - 1
        cpld_path = cpld_root + '/fan_' + str(x)
        super(fan, self).__init__(path,         \
                                  self.fan_fmt,  \
                                  0,            \
                                  cpld_path,    \
                                  ro)

    def decoder(self, s, t):
        if s == 'checksum':
            t = binascii.b2a_hex(t).upper()
            # Invert the bytes for endianess. This then matches the value
            # generated by dell-crc.
            t = struct.unpack(">H", struct.pack("<H", int(t,16)))[0]
            i = hex(t)
        elif s == 'header':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        status = super(fan, self).check_status()

        # each device may have a different status string, decide what
        # translates to 'ok' here.
        if status.startswith('ok'):
            return 'ok'
        else:
            return 'not present'

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 2

    def checksum_type(self):
        return 'dell-crc'
