#!/bin/sh

#
# Copyright 2015,2016,2018 Cumulus Networks, Inc.  All rights reserved
# All rights reserved.
#

# Invoked by /etc/init.d/hw_init

# Determine the fan configuration for multiple Dell S6010 SKUs and
# setup the fan related configuration files accordingly.

# The fan configuration files need only be setup once after a fresh
# install.

# Cache previously detected fan configuration
CACHE_DIR=/run/cache/cumulus/fan_config
CACHE_CONF="$CACHE_DIR/state"

SENSORS_CONF=/etc/sensors.d/dell_s6010_c2538.conf

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

detect_fans()
{
    # f2b_part_nums = ['083R0P', '01YRKK', '0N53X1', '036DN0']
    # b2f_part_nums = ['0FHD1H', '03H3JY', '0Y9TW5', '0K41TC']
    
    part_num=$(decode-syseeprom |grep 'Part Number'|awk '{print $5}')

    case "$part_num" in
    083R0P|01YRKK|0N53X1|036DN0)
        # F2B part nums
        FAN_CONFIG="F2B"
        ;;
    0FHD1H|03H3JY|0Y9TW5|0K41TC)
        # B2F part nums
        FAN_CONFIG="B2F"
        ;;
    *)
        # Default to B2F
        FAN_CONFIG="B2F"
        ;;
    esac
    # Store result in the cache
    mkdir -p $CACHE_DIR || {
        log_failure_msg "Problems making cache dir: $CACHE_DIR"
    }
    echo $FAN_CONFIG > $CACHE_CONF || {
        log_failure_msg "Problems writing cache file: $CACHE_CONF"
    }
}

# make_sensors_conf() -- Based on the fan type generate an appropriate
# sensors configuration file.
make_sensors_conf()
{
    config=$1

    # Emit common sensors config file text
    mkdir -p $(dirname $SENSORS_CONF)

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S6010 C2538 F2B
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
# On Dell S6010, U269, U270, U54, U245, U334 and U57 sensors control thermal hardware shutdown.
# For these sesnors, when the temperature hits the temp1_max value, the box will
# undergo thermal hardware shutdown. The box will power up  once the temperature
# lowers to temp1_max_hyst.
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"

#U269: tmp75-i2c-11-4b: Board Sensor near Front Left Vent
chip "tmp75-i2c-11-4b"
     set temp1_max 62
     set temp1_max_hyst 52

#U270: tmp75-i2c-11-4a: Board Sensor near Networking ASIC
chip "tmp75-i2c-11-4a"
     set temp1_max 72
     set temp1_max_hyst 62

#U54: tmp75-i2c-11-4c: Board Sensor near Front Right Vent
chip "tmp75-i2c-11-4c"
     set temp1_max 60
     set temp1_max_hyst 50

#U245: tmp75-i2c-11-4e: Board Sensor near QSFP
chip "tmp75-i2c-11-4e"
     set temp1_max 66
     set temp1_max_hyst 56

#U334: tmp75-i2c-11-4f: Board Sensor near Fan
chip "tmp75-i2c-11-4f"
     set temp1_max 64
     set temp1_max_hyst 54

#U57: tmp75-i2c-10-4d: Board Sensor near CPU
chip "tmp75-i2c-10-4d"
     set temp1_max 66
     set temp1_max_hyst 56

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-*"
     set temp1_max 55
     ignore temp2

# Fan sensors
chip "emc2305-*"

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# Hot swap controller
chip "ltc4215-*"
     ignore in1

EOF
    else
        # Use back-to-front configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S6010 C2538 B2F
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
# On Dell S6010, U278, U245, U244, U254, U288, U277 and U57 sensors control thermal hardware shutdown.
# For these sesnors, when the temperature hits the temp1_max value, the box will
# undergo thermal hardware shutdown. The box will power up  once the temperature
# lowers to temp1_max_hyst.
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"

#U269: tmp75-i2c-11-4b: Board Sensor near Front Left Vent
chip "tmp75-i2c-11-4b"
     set temp1_max 60
     set temp1_max_hyst 50

#U270: tmp75-i2c-11-4a: Board Sensor near Networking ASIC
chip "tmp75-i2c-11-4a"
     set temp1_max 79
     set temp1_max_hyst 69

#U54: tmp75-i2c-11-4c: Board Sensor near Front Right Vent
chip "tmp75-i2c-11-4c"
     set temp1_max 63
     set temp1_max_hyst 53

#U245: tmp75-i2c-11-4e: Board Sensor near QSFP
chip "tmp75-i2c-11-4e"
     set temp1_max 71
     set temp1_max_hyst 61

#U334: tmp75-i2c-11-4f: Board Sensor near Fan
chip "tmp75-i2c-11-4f"
     set temp1_max 64
     set temp1_max_hyst 54

#U57: tmp75-i2c-10-4d: Board Sensor near CPU
chip "tmp75-i2c-10-4d"
     set temp1_max 65
     set temp1_max_hyst 55

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-*"
     set temp1_max 55
     ignore temp2

# Fan sensors
chip "emc2305-*"

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# Hot swap controller
chip "ltc4215-*"
     ignore in1

EOF
    fi
}

# Main program starts here

if [ -r $CACHE_CONF ] ; then
    FAN_CONFIG=$(cat $CACHE_CONF)
    if [ $FAN_CONFIG != "F2B" ] && [ $FAN_CONFIG != "B2F" ] ; then
        log_failure_msg "Unexpected fan config: $FAN_CONFIG"
        exit 1
    fi
else
    detect_fans
    make_sensors_conf $FAN_CONFIG
fi
