#!/bin/sh

#
# Copyright (c) 2017,2018 Cumulus Networks, Inc.  All rights reserved.
#

# Invoked by /etc/init.d/hw_init

# Determine the fan configuration for multiple Dell S4148 SKUs and
# setup the fan related configuration files accordingly.

# The fan configuration files need only be setup once after a fresh
# install.

# Cache previously detected fan configuration
CACHE_DIR=/run/cache/cumulus/fan_config
CACHE_CONF="$CACHE_DIR/state"

SENSORS_CONF=/etc/sensors.d/dell_s4148_c2338.conf

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

detect_fans()
{
    # f2b_part_nums = ['0264Y6', '0FPJ4D', '0PJV6G']
    # b2f_part_nums = ['0401J0', '0XF6VC', '0T3D30']
    part_num=$(decode-syseeprom |grep 'Part Number'|awk '{print $5}')
    case "$part_num" in
    0264Y6|0FPJ4D|0PJV6G)
        # F2B part nums
        FAN_CONFIG="F2B"
        ;;
    0401J0|0XF6VC|0T3D30)
        # B2F part nums
        FAN_CONFIG="B2F"
        ;;
    *)
        # Default to B2F
        FAN_CONFIG="B2F"
        ;;
    esac
    # Store result in the cache
    mkdir -p $CACHE_DIR || {
        log_failure_msg "Problems making cache dir: $CACHE_DIR"
    }
    echo $FAN_CONFIG > $CACHE_CONF || {
        log_failure_msg "Problems writing cache file: $CACHE_CONF"
    }
}

# make_sensors_conf() -- Based on the fan type generate an appropriate
# sensors configuration file.
make_sensors_conf()
{
    config=$1

    # Emit common sensors config file text
    mkdir -p $(dirname $SENSORS_CONF)

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S4148 C2338 F2B
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
#
# On Dell S4148, the U359, U360, U54, U245, U334, and U57 sensors control
# thermal hardware shutdown.  For these sesnors, when the temperature hits the
# temp1_max value, the box will undergo thermal hardware shutdown. The box
# will power up  once the temperature lowers to temp1_max_hyst.

bus "i2c-0" "SMBus I801 adapter at f000"
bus "i2c-1" "SMBus iSMT adapter at dff14000"
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"
bus "i2c-12" "i2c-1-mux (chan_id 2)"
bus "i2c-13" "i2c-1-mux (chan_id 3)"
bus "i2c-14" "i2c-1-mux (chan_id 4)"
bus "i2c-15" "i2c-1-mux (chan_id 5)"
bus "i2c-16" "i2c-1-mux (chan_id 6)"
bus "i2c-17" "i2c-1-mux (chan_id 7)"

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# U359: tmp75-i2c-13-4b: Board sensor near left of front vents
chip "tmp75-i2c-13-4b"
     label temp1 "board sensor near left of front vents"
     set temp1_max 63
     set temp1_max_hyst 53

# U360: tmp75-i2c-13-4a: Board sensor near Maverick
chip "tmp75-i2c-13-4a"
     label temp1 "board sensor near switch chip"
     set temp1_max 66
     set temp1_max_hyst 56

# U54: tmp75-i2c-13-4c: Board sensor near right of front vents
chip "tmp75-i2c-13-4c"
     label temp1 "board sensor near right of front vents"
     set temp1_max 61
     set temp1_max_hyst 51

# U245: tmp75-i2c-13-4e: Board sensor near QSFP
chip "tmp75-i2c-13-4e"
     label temp1 "board sensor near QSFP"
     set temp1_max 64
     set temp1_max_hyst 54

# U334: tmp75-i2c-13-4f: Board sensor near DC fan
chip "tmp75-i2c-13-4f"
     label temp1 "board sensor near DC fan"
     set temp1_max 62
     set temp1_max_hyst 52

# U57: tmp75-i2c-10-4d: Board sensor near CPU
chip "tmp75-i2c-10-4d"
     label temp1 "board sensor near CPU"
     set temp1_max 66
     set temp1_max_hyst 56

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-i2c-14-58"
     label fan1 "psu1 fan"
     label temp1 "psu1 inlet temp"
     label temp2 "psu1 heatsink temp"
     set temp1_max 65
     set temp2_max 65

chip "dps460-i2c-14-59"
     label fan1 "psu1 fan"
     label temp1 "psu1 inlet temp"
     label temp2 "psu1 heatsink temp"
     set temp1_max 65
     set temp2_max 65

# Fan sensors
chip "emc2305-i2c-11-4d"
     ignore fan5

# DIMM Temp Sensor
chip "jc42-i2c-0-1a"
     ignore temp1

EOF
    else
        # Use back-to-front configuration
	cat <<EOF > $SENSORS_CONF
# libsensors configuration file for DELL S4148 C2338 F2B
# ------------------------------------------------------
#
# WARNING: Please do not edit this file, unless you know what you are doing.
#
# On Dell S4148, the U359, U360, U54, U245, U334, and U57 sensors control
# thermal hardware shutdown.  For these sesnors, when the temperature hits the
# temp1_max value, the box will undergo thermal hardware shutdown. The box
# will power up  once the temperature lowers to temp1_max_hyst.

bus "i2c-0" "SMBus I801 adapter at f000"
bus "i2c-1" "SMBus iSMT adapter at dff14000"
bus "i2c-10" "i2c-1-mux (chan_id 0)"
bus "i2c-11" "i2c-1-mux (chan_id 1)"
bus "i2c-12" "i2c-1-mux (chan_id 2)"
bus "i2c-13" "i2c-1-mux (chan_id 3)"
bus "i2c-14" "i2c-1-mux (chan_id 4)"
bus "i2c-15" "i2c-1-mux (chan_id 5)"
bus "i2c-16" "i2c-1-mux (chan_id 6)"
bus "i2c-17" "i2c-1-mux (chan_id 7)"

# Intel CPU die Temp sensor
# sensor parameters are read-only.
chip "coretemp-*"

# U359: tmp75-i2c-13-4b: Board sensor near left of front vents
chip "tmp75-i2c-13-4b"
     label temp1 "board sensor near left of front vents"
     set temp1_max 66
     set temp1_max_hyst 56

# U360: tmp75-i2c-13-4a: Board sensor near Maverick
chip "tmp75-i2c-13-4a"
     label temp1 "board sensor near switch chip"
     set temp1_max 79
     set temp1_max_hyst 69

# U54: tmp75-i2c-13-4c: Board sensor near right of front vents
chip "tmp75-i2c-13-4c"
     label temp1 "board sensor near right of front vents"
     set temp1_max 65
     set temp1_max_hyst 55

# U245: tmp75-i2c-13-4e: Board sensor near QSFP
chip "tmp75-i2c-13-4e"
     label temp1 "board sensor near QSFP"
     set temp1_max 68
     set temp1_max_hyst 58

# U334: tmp75-i2c-13-4f: Board sensor near DC fan
chip "tmp75-i2c-13-4f"
     label temp1 "board sensor near DC fan"
     set temp1_max 62
     set temp1_max_hyst 52

# U57: tmp75-i2c-10-4d: Board sensor near CPU
chip "tmp75-i2c-10-4d"
     label temp1 "board sensor near CPU"
     set temp1_max 63
     set temp1_max_hyst 53

# PSU1 and PSU2: 2 temp sensors and a fan sensor
chip "dps460-i2c-14-58"
     label fan1 "psu1 fan"
     label temp1 "psu1 inlet temp"
     label temp2 "psu1 heatsink temp"
     set temp1_max 65
     set temp2_max 65

chip "dps460-i2c-14-59"
     label fan1 "psu1 fan"
     label temp1 "psu1 inlet temp"
     label temp2 "psu1 heatsink temp"
     set temp1_max 65
     set temp2_max 65

# Fan sensors
chip "emc2305-i2c-11-4d"
     ignore fan5

# DIMM Temp Sensor
chip "jc42-i2c-0-1a"
     ignore temp1

EOF
    fi
}

# Main program starts here

if [ -r $CACHE_CONF ] ; then
    FAN_CONFIG=$(cat $CACHE_CONF)
    if [ $FAN_CONFIG != "F2B" ] && [ $FAN_CONFIG != "B2F" ] ; then
        log_failure_msg "Unexpected fan config: $FAN_CONFIG"
        exit 1
    fi
else
    detect_fans
    make_sensors_conf $FAN_CONFIG
fi
