#! /usr/bin/python
# Copyright 2019 Cumulus Networks , Inc.

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import binascii
    import os
    import subprocess
    import tempfile
    import eeprom_base
    import eeprom_tlvinfo
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

def write_flash_eeprom(e, dev_path):
    '''
    Utility function to write the EEPROM data to a MTD NAND flash
    partition.
    '''
    (fd, tmp_path) = tempfile.mkstemp()
    F = os.fdopen(fd, 'wb')
    F.write(e)
    F.close()

    # flash contents to board_eeprom
    cmd = []
    cmd.append('/usr/sbin/nandwrite')
    cmd.append('-p')
    cmd.append(dev_path)
    cmd.append(tmp_path)
    try:
        p = subprocess.check_output((cmd), stderr=subprocess.STDOUT)
    except:
        # clean up
        os.unlink(tmp_path)
        raise RuntimeError('Unable to run "%s"' % cmd[0])

    # clean up
    os.unlink(tmp_path)

# Current TLV based EEPROM format
class board(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x40000

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path, 0, "", ro, self.EEPROM_MAX_SIZE)

    def write_eeprom(self, e):
        write_flash_eeprom(e, self.p)

#
# power supply unit
#
class psu(eeprom_base.eepromDecoder):

    ps_fmt = [ ('order_number', 's', 32),
               ('description', 's', 32),
               ('serial_num', 's', 16),
               ('mfg_pn', 's', 16),
               ('mfg_rev', 's', 4),
               ('customer_pn', 's', 16),
               ('mfg_date', 's', 16),
               ('mfg_location', 's', 16),
               ('RoHs compliance', 's', 16),
               ('mfg_deviations', 's', 32),
               ('odm_pn', 's', 32),
               ('checksum', 'c', 1)
             ]

    def __init__(self, name, path, cpld_root, ro):
        cpld_root = '/sys/bus/i2c/devices/0-0028'
        cpld_path = cpld_root + '/psu_pwr' + name.split('psu')[1] + '_present'
        super(psu, self).__init__(path,         \
                                  self.ps_fmt,  \
                                  0,            \
                                  cpld_path,    \
                                  ro)

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        status = super(psu, self).check_status()

        # each device may have a different status string, decide what
        # translates to 'ok' here.
        if status.startswith('1'):
            return 'ok'
        else:
            return 'not present'

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 1

    def checksum_type(self):
        return '2s-complement'
