#! /usr/bin/python
# Copyright 2015 Cumulus Networks LLC, all rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import binascii
    import time
    import optparse
    import warnings
    import os
    import sys
    import subprocess
    import tempfile
    import eeprom_base
    import eeprom_tlvinfo
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

# The DNI-3448 has two EEPROM formats, an older fixed field format and
# the current TLV based format.

def write_flash_eeprom(e, dev_path):
    '''
    Utility function to write the EEPROM data to a MTD NOR flash
    partition.
    '''
    (fd, tmp_path) = tempfile.mkstemp()
    F = os.fdopen(fd, 'wb')
    F.write(e)
    F.close()

    # flash contents to board_eeprom
    cmd = []
    cmd.append('/usr/sbin/flashcp')
    cmd.append(tmp_path)
    cmd.append(dev_path)
    try:
        p = subprocess.Popen((cmd), stdout=subprocess.PIPE,
                             shell=False, stderr=subprocess.STDOUT)
        txt = p.communicate()[0]
        p.wait()
    except:
        # clean up
        os.unlink(tmp_path)
        raise RuntimeError('Unable to run "%s"' % cmd[0])

    # clean up
    os.unlink(tmp_path)

    if p.returncode != 0:
        cmd_str = ' '.join(cmd)
        raise RuntimeError('Problems running "%s"...\n%s' % (cmd_str, txt))

# Current TLV based EEPROM format
class board_new_format(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x40000

    def __init__(self, name, path, cpld_root, ro):
        super(board_new_format, self).__init__(path, 0, "", ro, self.EEPROM_MAX_SIZE)

    def write_eeprom(self, e):
        write_flash_eeprom(e, self.p)
        self.write_cache(e)

# Older, fixed field EEPROM format
class board_old_format(eeprom_base.eepromDecoder):
    brd_fmt = [ ('id', 's', 4),
                ('serial_num', 's', 12),
                ('errata_level', 's', 5),
                ('build_date', 'C', 6),
                ('burn', 'x', 1),
                ('version', 'C', 4),
                ('burn', 'x', 8),
                ('burn', 'x', 2),
                ('burn', 'x', 1),
                ('burn', 'x', 21),
                ('mac_count', 'C', 1),
                ('mac_flag', 's', 1),
                ('macaddr', 'C', 6),
                ('burn', 'x', 42),
                ('crc', 'C', 4)
              ]

    def __init__(self, name, path, cpld_root, ro):
        super(board_old_format, self).__init__(path,  \
                                    self.brd_fmt,     \
                                    0,                \
                                    '',               \
                                    ro)

    def base_mac_addr(self, e):
        (is_valid, valid_crc) = self.is_checksum_valid(e)
        if is_valid is False:
            return super(board_old_format, self).base_mac_addr(e)

        loc = 0
        for I in self.brd_fmt:
            end = loc + I[2]
            t = e[loc:end]
            loc = end
            if I[0] != 'macaddr':
                continue
            else:
                mac = ":".join([binascii.b2a_hex(T) for T in list(t)])
                return mac
        raise RuntimeError("did not find mac address")


    def switchaddrrange(self, e):
        # this is a cluster-f, currently we allocate all 3448 addresses from the
        # cumulus pool, which is allocated in 53 entry blocks
        return 53

    def serial_number_str(self, e):
        valid, _ = self.is_checksum_valid(e)
        if not valid:
            return super(board_old_format, self).serial_number_str(e)
        return super(board_old_format, self).find_field(e, 'serial_num')

    def decoder(self, s, t):
        if s == 'version':
            i = ".".join([str(ord(T)) for T in list(t)])
        elif s == 'build_date':
            i = "20" + str(ord(t[0])) + "/" + \
                str(ord(t[1])).rjust(2, '0') + "/" + \
                str(ord(t[2])).rjust(2, '0') + " " + \
                str(ord(t[3])).rjust(2, '0') + "/" + \
                str(ord(t[4])).rjust(2, '0') + "/" + \
                str(ord(t[5])).rjust(2, '0')
        elif s == 'mac_count':
            i = str(ord(t[0]))
        elif s == 'macaddr':
            i = ":".join([binascii.b2a_hex(T) for T in list(t)]).upper()
        elif s == 'crc':
            i = "0x" + binascii.b2a_hex(t).upper()

        return i

    def encoder(self, I, v):
        l = ''
        try:
            if I[0] == 'id':
                l += 'NXID'
            elif I[0] == 'version':
                errstr = 'x.x.x.x where x is a digit'
                if len(v.split('.')) != 4:
                    raise
                l += ''.join(chr(int(C)) for C in v.split('.'))
            elif I[0] == 'build_date':
                errstr = 'YYYY/MM/DD hh/mm/ss'
                [f1, j, f2] = v.rstrip('\n\r').partition(' ')
                [y, m ,d] = f1.split('/')
                [hh, mm, ss] = f2.strip().split('/')
                if int(m) not in range(1,13) or int(d) not in range(1,32) or \
                   int(y) < 2000 or int(hh) > 23 or int(mm) > 59 or int(ss) > 59:
                    raise
                l += chr(int(y) - 2000) + chr(int(m)) + chr(int(d)) + \
                     chr(int(hh)) + chr(int(mm)) + chr(int(ss))
            elif I[0] == 'mac_count':
                errstr = 'numeric'
                l += chr(int(v))
            elif I[0] == 'macaddr':
                errstr = 'xx:xx:xx:xx:xx:xx  where xx is alpha-numeric'
                if len(v.split(':')) != 6:
                    raise
                mac = v.replace(':', '', 5)
                l += binascii.a2b_hex(mac)
            elif I[1] == 's':
                # left adjust the field to the defined length
                l += v.ljust(I[2])[:I[2]]
        except:
            sys.stderr.write("Error: '" + I[0] + "' correct format is " + errstr + "\n")
            exit(0)

        return l

    def is_checksum_field(self, I):
        return I[0] == 'crc'

    def checksum_field_size(self):
        return 4

    def checksum_type(self):
        return 'crc32'

    def write_eeprom(self, e):
        write_flash_eeprom(e, self.p)
        self.write_cache(e)

def board(name, path, cpld_root, ro):
    '''
    Look to see which type of board EEPROM format we have.  Based
    on that we return an instance of the appropriate class.

    Check for a valid TLV header.  If found assume we have the TLV
    based format.  Otherwise assume it is the older format.
    '''

    # Try new format first
    new_format = board_new_format(name, path, cpld_root, ro)
    F = new_format.open_eeprom()
    h = F.read(new_format._TLV_INFO_HDR_LEN)
    if len(h) != new_format._TLV_INFO_HDR_LEN:
        raise RuntimeError("expected to read %d bytes from %s, " % \
                           (new_format._TLV_INFO_HDR_LEN, path) + \
                           "but only read %d" % (len(h)))
    F.close()
    if new_format.is_valid_tlvinfo_header(h):
        return new_format

    # Try old format
    old_format = board_old_format(name, path, cpld_root, ro)
    e = old_format.read_eeprom()
    (is_valid, valid_crc) = old_format.is_checksum_valid(e)
    if is_valid:
        return old_format

    # Unknown format or corrupt.  Fallback to the new format so that
    # writing will use the new format.
    return new_format

#
# power supply unit
#
class psu(eeprom_base.eepromDecoder):

    ps_fmt = [ ('order_number', 's', 32),
               ('description', 's', 32),
               ('serial_num', 's', 16),
               ('mfg_pn', 's', 16),
               ('mfg_rev', 's', 4),
               ('customer_pn', 's', 16),
               ('mfg_date', 's', 16),
               ('mfg_location', 's', 16),
               ('RoHs compliance', 's', 16),
               ('mfg_deviations', 's', 32),
               ('odm_pn', 's', 32),
               ('checksum', 'c', 1)
             ]

    def __init__(self, name, path, cpld_root, ro):
        cpld_root = '/sys/bus/i2c/devices/0-0028'
        cpld_path = cpld_root + '/psu_pwr' + name.split('psu')[1] + '_present'
        super(psu, self).__init__(path,         \
                                  self.ps_fmt,  \
                                  0,            \
                                  cpld_path,    \
                                  ro)

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        status = super(psu, self).check_status()

        # each device may have a different status string, decide what
        # translates to 'ok' here.
        if status.startswith('1'):
            return 'ok'
        else:
            return 'not present'

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 1

    def checksum_type(self):
        return '2s-complement'
