#! /usr/bin/python
# Copyright 2012 Cumulus Networks LLC, all rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import binascii
    import time
    import optparse
    import warnings
    import os
    import sys
    import eeprom_base
    import eeprom_tlvinfo
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

# The DNI-7448 has two EEPROM formats, an older fixed field format and
# the current TLV based format.

# Current TLV based EEPROM format
class board_new_format(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_OFFSET   = 0x300
    EEPROM_MAX_SIZE = 0x100
    
    def __init__(self, name, path, cpld_root, ro):
        super(board_new_format, self).__init__(path,
                                               self.EEPROM_OFFSET,
                                               "",
                                               ro,
                                               self.EEPROM_MAX_SIZE)

class board_new_format_legacy(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_OFFSET        = 0x0
    EEPROM_MAX_SIZE      = 0x100
    
    def __init__(self, name, path, cpld_root, ro):
        super(board_new_format_legacy, self).__init__(path,
                                                      self.EEPROM_OFFSET,
                                                      "",
                                                      ro,
                                                      self.EEPROM_MAX_SIZE)

# Older, fixed field EEPROM format
class board_old_format(eeprom_base.eepromDecoder):
    brd_fmt = [ ('vendor_id', 's', 32),
                ('country_code', 's', 32),
                ('date_code', 's', 32),
                ('serial_number', 's', 32),
                ('part_number', 's', 32),
                ('revision', 's', 32),
                ('order_number', 's', 32),
                ('component_type', 's', 32),
                ('component_id', 's', 32),
                ('start_mac', 'C', 6),
                ('end_mac', 'C', 6),
                ('burn', 'x', 20),
                ('checksum', 'C', 1),
              ]

    def __init__(self, name, path, cpld_root, ro):
        super(board_old_format, self).__init__(path,  \
                                    self.brd_fmt,     \
                                    0,                \
                                    '',               \
                                    ro)

    def findmacfield(self, e, field):
        loc = 0
        mac = None
        for I in self.brd_fmt:
            end = loc + I[2]
            t = e[loc:end]
            loc = end
            if I[0] != field:
                continue
            else:
                mac = ":".join([binascii.b2a_hex(T) for T in list(t)])
                return mac

    def base_mac_addr(self, e):
        (is_valid, valid_crc) = self.is_checksum_valid(e)
        if is_valid is False:
            return super(board_old_format, self).base_mac_addr(e)
        mac = self.findmacfield(e, 'start_mac')
        if mac is not None :
            return mac
        else:
            return super(board_old_format, self).base_mac_addr(e)

    def switchaddrrange(self, e):
        (is_valid, valid_crc) = self.is_checksum_valid(e)
        if is_valid is False:
            return 3 # assume there are 3 mac addresses allocated when we don't know
        mac1 = self.findmacfield(e, 'start_mac')
        mac2 = self.findmacfield(e, 'end_mac')
        if mac1 is not None and mac2 is not None :
            return self.diff_mac(mac1, mac2)
        else:
            return 3

    def serial_number_str(self, e):
        valid, _ = self.is_checksum_valid(e)
        if not valid:
            return super(board_old_format, self).serial_number_str(e)
        return super(board_old_format, self).find_field(e, 'serial_number')

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        elif s == 'start_mac' or s == 'end_mac':
            i = ":".join([binascii.b2a_hex(T) for T in list(t)])
        else:
            i = t
        return i

    def encoder(self, I, v):
        l = ''
        try:
            if I[1] == 's':
                # left adjust the field to the defined length
                l += v.ljust(I[2], '\x00')[:I[2]]
            if I[0] == 'start_mac' or I[0] == 'end_mac':
                errstr = 'xx:xx:xx:xx:xx:xx  where xx is alpha-numeric'
                if len(v.split(':')) != 6:
                    raise
                mac = v.replace(':', '', 5)
                l += binascii.a2b_hex(mac)
        except:
            sys.stderr.write("Error: '" + I[0] + "' correct format is " + errstr + "\n")
            exit(0)

        return l

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 1

    def checksum_type(self):
        return '2s-complement'

def board(name, path, cpld_root, ro):
    '''
    Look to see which type of board EEPROM format we have.  Based
    on that we return an instance of the appropriate class.

    Check for a valid TLV header.  If found assume we have the TLV based
    format.  Otherwise assume it is the older format.
    '''

    # Try new format first
    new_format = board_new_format(name, path, cpld_root, ro)
    h = new_format.read_eeprom_bytes(new_format._TLV_INFO_HDR_LEN)
    if new_format.is_valid_tlvinfo_header(h):
        return new_format

    # Try new format at legacy offset second
    new_format_legacy = board_new_format_legacy(name, path, cpld_root, ro)
    h = new_format_legacy.read_eeprom_bytes(new_format_legacy._TLV_INFO_HDR_LEN)
    if new_format_legacy.is_valid_tlvinfo_header(h):
        return new_format_legacy

    # Try old format
    old_format = board_old_format(name, path, cpld_root, ro)
    e = old_format.read_eeprom()
    (is_valid, valid_crc) = old_format.is_checksum_valid(e)
    if is_valid:
        return old_format

    # Unknown format or corrupt.  Fallback to the new format so that
    # writing will use the new format.
    return new_format

#
# power supply unit
#

class psu_dell_format(eeprom_base.eepromDecoder):
    ps_dell_fmt = [ ('burn', 'h', 6),
                    ('vendor_id', 's', 2),
                    ('country_code', 's', 2),
                    ('date', 'd', 8),
                    ('serial_number', 's', 14),
                    ('part_num', 's', 12),
                    ('prod_rev', 's', 2),
                    ('prod_order_num', 's', 25),
                    ('type_id', 'x', 2),
                    ('module_id', 's', 2),
                    ('burn', 's', 19),
                    ('checksum', 'c', 2)
                    ]

    def __init__(self, name, path, cpld_root, ro):
        cpldpath = cpld_root + '/psu_pwr' + name.split('psu')[1]
        super(psu_dell_format, self).__init__(path,
                                  self.ps_dell_fmt,
                                  0,
                                  cpldpath,
                                  ro)
    def is_valid_header(self):
        F = self.open_eeprom()
        F.seek(self.s)
        o = F.read(2)
        i = binascii.b2a_hex(o).upper()
        if i == '3A29':
            return True
        return False

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        elif s == 'header':
            i = "0x" + binascii.b2a_hex(t).upper()
        elif s == 'date':
            i = t[0:2] + '/'+ t[2:4] + '/' + t[4:8]
        elif s == 'type_id':
            x = int(t)
            if x == 1:
                i = 'A/C Normal'
            elif x == 2:
                i = 'A/C Reverse'
            elif x == 3:
                i = 'D/C Normal'
            elif x == 4:
                i = 'D/C Reverse'
            else:
                i = 'Unknown(%d)' %x
        else:
            i = t
        return i

    def check_status(self):
        status = super(psu_dell_format, self).check_status()
        # each device may have a different status string, decide what
        # translates to 'ok' here.
        if status.startswith('installed'):
            return 'ok'
        else:
            return status

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 2

    def checksum_type(self):
        return 'dell-crc'

class psu_dni_format(eeprom_base.eepromDecoder):

    ps_dni_fmt = [ ('order_number', 's', 32),
               ('description', 's', 32),
               ('serial_num', 's', 16),
               ('mfg_pn', 's', 16),
               ('customer_pn', 's', 16),
               ('mfg_date', 's', 16),
               ('mfg_location', 's', 16),
               ('RoHs compliance', 's', 16),
               ('mfg_deviations', 's', 32),
               ('odm_pn', 's', 32),
               ('fan_air_flow_id', 'c', 1),
               ('checksum', 'c', 1)
               ]

    def __init__(self, name, path, cpld_root, ro):
        cpldpath = cpld_root + '/psu_pwr' + name.split('psu')[1]
        super(psu_dni_format, self).__init__(path,         \
                                  self.ps_dni_fmt,  \
                                  0,            \
                                  cpldpath,     \
                                  ro)

    def decoder(self, s, t):
        if s == 'checksum':
            i = "0x" + binascii.b2a_hex(t).upper()
        else:
            i = t
        return i

    def check_status(self):
        status = super(psu_dni_format, self).check_status()
        # each device may have a different status string, decide what
        # translates to 'ok' here.
        if status.startswith('installed'):
            return 'ok'
        else:
            return status

    def is_checksum_field(self, I):
        return I[0] == 'checksum'

    def checksum_field_size(self):
        return 1

    def checksum_type(self):
        return '2s-complement'

def psu(name, path, cpld_root, ro):
    cpld_root = '/sys/devices/ffe05000.localbus/ffdf0000.cpld'
    psu_dell = psu_dell_format(name, path, cpld_root, ro)
    if psu_dell.check_status() != 'ok':
        # The init code will detect it as absent and exit
        return psu_dell
    if psu_dell.is_valid_header():
        return psu_dell
    psu_dni = psu_dni_format(name, path, cpld_root, ro)
    return psu_dni
