#!/bin/sh

#
# Copyright 2013,2015,2018 Cumulus Networks, Inc.
# All rights reserved.
#

# Invoked by /etc/init.d/hw_init

# Determine the fan configuration for multiple Quanta LB9 SKUs and
# setup the fan related configuration files accordingly.
#
# |   SKU     |   Air Flow    |      Adapter     |   Fans
# ============+===============+==================+===========
# 1LB9BZZ0STQ   Front-to-Back   adt7470-i2c-5-2c   1, 2, 3, 4
# 1LB9BZZ0STR   Back-to-Front   adt7470-i2c-6-2f   1, 2, 3, 4
#
# First we try reading the EEPROM.  The 'part number' node should tell
# us what type of system we have.  Failing that we read all 8 fan
# tachometers to determine which type of system we have.  Unconnected
# fans return a zero tachometer reading.
#
# The fan configuration files need only be setup once after a fresh
# install.

# Cache previously detected fan configuration
CACHE_DIR=/run/cache/cumulus/fan_config
CACHE_CONF="$CACHE_DIR/state"

# The following config file depend on the fan devices
SENSORS_CONF=/etc/sensors.d/quanta.conf
FAN_MONITOR_CONF=/etc/fan_monitor/fan_monitor.cfg
PWMD_CONF=/etc/pwmd.conf

[ "$1" = "start" ] || exit 0

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

F2B_CONTROL_SYSFS="/sys/bus/i2c/devices/5-002c"
B2F_CONTROL_SYSFS="/sys/bus/i2c/devices/6-002f"

detect_fans()
{
    # First try the EEPROM
    # Part Number: 1LB9BZZ0STQ -- F2B
    # Part Number: 1LB9BZZ0STR -- B2F
    part_num=$(decode-syseeprom |grep 'Part Number'|awk '{print $5}')
    if [ "$part_num" = "1LB9BZZ0STQ" ] ; then
        FAN_CONFIG="F2B"
    elif [ "$part_num" = "1LB9BZZ0STR" ] ; then
        FAN_CONFIG="B2F"
    else
        # Detect what fans we do have

        # Read fan values, keeping a count of how many fan RPMs are greater
        # than zero.
        F2B_CNT=0
        B2F_CNT=0
        IDX=1
        for ctrl in F2B B2F ; do
            while [ $IDX -le 4 ] ; do
                eval fan_input=\""\$${ctrl}_CONTROL_SYSFS/fan${IDX}_input"\"
                [ -r $fan_input ] || {
                    log_failure_msg "Unable to find $ctrl fan${IDX}: $fan_input"
                    exit 1
                }
                val=$(cat $fan_input)
                if [ $val -gt 0 ] ; then
                    eval ${ctrl}_CNT=$(( ${ctrl}_CNT + 1 ))
                fi
                IDX=$(( $IDX + 1 ))
            done
        done

        if [ $F2B_CNT -eq 0 ] && [ $B2F_CNT -eq 0 ] ; then
            log_failure_msg "All fans read back zero RPM"
            exit 1
        fi

        # Use whichever count is greater.  If there is a tie then go with
        # front-to-back.
        if [ $F2B_CNT -ge $B2F_CNT ] ; then
            FAN_CONFIG="F2B"
        else
            FAN_CONFIG="B2F"
        fi
    fi

    # Store result in the cache
    mkdir -p $CACHE_DIR || {
        log_failure_msg "Problems making cache dir: $CACHE_DIR"
    }
    echo $FAN_CONFIG > $CACHE_CONF || {
        log_failure_msg "Problems writing cache file: $CACHE_CONF"
    }

}

# Construct common config file fragments
PLATFORM="Quanta LB9"
MIN_RPM=2500
MAX_RPM=14000
MIN_TEMP=5
MAX_TEMP=80

# make_sensors_conf() -- Based on the fan type generate an appropriate
# sensors configuration file.
make_sensors_conf()
{
    config=$1

    # Emit common sensors config file text
    mkdir -p $(dirname $SENSORS_CONF)

    # Common config for adt7470-i2c-*-2c chip
    (cat <<EOF
# libsensors configuration file for $PLATFORM

chip "adt7470-i2c-*-2c"

     set temp1_min $MIN_TEMP
     set temp1_max 50
     set temp2_min $MIN_TEMP
     set temp2_max $MAX_TEMP
     set temp3_min $MIN_TEMP
     set temp3_max 58
     set temp4_min $MIN_TEMP
     set temp4_max 58
     set temp5_min $MIN_TEMP
     set temp5_max $MAX_TEMP

     # temps 6 through 10 are not wired up, ignore them.
     ignore temp6
     ignore temp7
     ignore temp8
     ignore temp9
     ignore temp10

EOF
    ) > $SENSORS_CONF

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
        (cat <<EOF

     set fan1_min $MIN_RPM
     set fan1_max $MAX_RPM
     set fan2_min $MIN_RPM
     set fan2_max $MAX_RPM
     set fan3_min $MIN_RPM
     set fan3_max $MAX_RPM
     set fan4_min $MIN_RPM
     set fan4_max $MAX_RPM

EOF
        ) >> $SENSORS_CONF
    else
        # Use back-to-front configuration
        (cat <<EOF
     # For systems with back-to-front air flow fans 1,2,3 and 4 of
     # this controller are unconnected.

     ignore fan1
     ignore fan2
     ignore fan3
     ignore fan4

EOF
        ) >> $SENSORS_CONF
    fi

    # Common config for adt7470-i2c-*-2f chip
    (cat <<EOF
chip "adt7470-i2c-*-2f"

     # temps 1 through 10 are not wired up, ignore them.
     ignore temp1
     ignore temp2
     ignore temp3
     ignore temp4
     ignore temp5
     ignore temp6
     ignore temp7
     ignore temp8
     ignore temp9
     ignore temp10
EOF
        ) >> $SENSORS_CONF

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
        (cat <<EOF
     # For systems with front-to-back air flow fans 1,2,3 and 4 of
     # this controller are unconnected.

     ignore fan1
     ignore fan2
     ignore fan3
     ignore fan4

EOF
        ) >> $SENSORS_CONF
    else
        # Use back-to-front configuration
        (cat <<EOF

     set fan1_min $MIN_RPM
     set fan1_max $MAX_RPM
     set fan2_min $MIN_RPM
     set fan2_max $MAX_RPM
     set fan3_min $MIN_RPM
     set fan3_max $MAX_RPM
     set fan4_min $MIN_RPM
     set fan4_max $MAX_RPM

EOF
        ) >> $SENSORS_CONF
    fi

}

# make_pwmd_conf() -- Based on the fan type generate an appropriate
# pwmd configuration file.
make_pwmd_conf()
{
    config=$1

    # Emit common pwmd config file text
    mkdir -p $(dirname $PWMD_CONF)
    (cat <<EOF
# pwmd configuration for $PLATFORM

[DEFAULT]
interval = 20
minpwm = 64
maxpwm = 255

# Temperature Sensors
monitor = adt7470-i2c-5-2c-temp1
          adt7470-i2c-5-2c-temp2
          adt7470-i2c-5-2c-temp3
          adt7470-i2c-5-2c-temp4
          adt7470-i2c-5-2c-temp5

# Fan specific configuration sections

EOF
    ) > $PWMD_CONF

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
        (cat <<EOF
[adt7470-i2c-5-2c-pwm1] 

[adt7470-i2c-5-2c-pwm2] 

[adt7470-i2c-5-2c-pwm3] 

[adt7470-i2c-5-2c-pwm4] 

EOF
        ) >> $PWMD_CONF
    else
        # Use back-to-front configuration
        (cat <<EOF
[adt7470-i2c-6-2f-pwm1] 

[adt7470-i2c-6-2f-pwm2] 

[adt7470-i2c-6-2f-pwm3] 

[adt7470-i2c-6-2f-pwm4] 

EOF
        ) >> $PWMD_CONF
    fi

}

# make_fan_monitor_conf() -- Based on the fan type generate an appropriate
# fan_monitor configuration file.
make_fan_monitor_conf()
{
    config=$1

    if [ $config = "F2B" ] ; then
        sensor="adt7470-i2c-5"
    else
        sensor="adt7470-i2c-6"
    fi

    # Emit common fan_monitor config file text
    mkdir -p $(dirname $FAN_MONITOR_CONF)
    (cat <<EOF
# Fan monitor configuration for $PLATFORM

[Control]

num_fans = 4
sensor_type = $sensor

[Fans]

fan1_high = $MAX_RPM

fan2_high = $MAX_RPM

fan3_high = $MAX_RPM

fan4_high = $MAX_RPM

EOF
    ) > $FAN_MONITOR_CONF

}

# Main program starts here

# Sanity checking:
[ -d $F2B_CONTROL_SYSFS ] || {
    log_failure_msg "Unable to find sensor chip: $F2B_CONTROL_SYSFS"
    exit 1
}
[ -d $B2F_CONTROL_SYSFS ] || {
    log_failure_msg "Unable to find sensor chip: $B2F_CONTROL_SYSFS"
    exit 1
}

if [ -r $CACHE_CONF ] ; then
    FAN_CONFIG=$(cat $CACHE_CONF)
    if [ $FAN_CONFIG != "F2B" ] && [ $FAN_CONFIG != "B2F" ] ; then
        log_failure_msg "Unexpected fan config: $FAN_CONFIG"
        exit 1
    fi
else
    detect_fans
fi

# The configuration files need only be created once.  If they already
# exist do not touch them.

# sensors config
[ -r $SENSORS_CONF ] || make_sensors_conf $FAN_CONFIG

# pwmd config
[ -r $PWMD_CONF ] || make_pwmd_conf $FAN_CONFIG

# fan monitor config
[ -r $FAN_MONITOR_CONF ] || make_fan_monitor_conf $FAN_CONFIG
