#! /usr/bin/python
# Copyright 2012 Cumulus Networks LLC, all rights reserved

#############################################################################
# Platform and model specific eeprom subclass, inherits from the base class,
# and provides the followings:
# - the eeprom format definition
# - specific encoder/decoder if there is special need
#

try:
    import exceptions
    import binascii
    import time
    import optparse
    import warnings
    import os
    import sys
    import eeprom_tlvinfo
#    import crcmod
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")
#
# base board
#
class board(eeprom_tlvinfo.TlvInfoDecoder):

    EEPROM_MAX_SIZE = 0x100

    def __init__(self, name, path, cpld_root, ro):
        super(board, self).__init__(path,             \
                                    0,                \
                                    '',               \
                                    ro,               \
                                    self.EEPROM_MAX_SIZE)

        # HEADER handling
        eeprom_tlvinfo.TlvInfoDecoder._TLV_HDR_ENABLED            = 0
        eeprom_tlvinfo.TlvInfoDecoder._TLV_INFO_MAX_LEN           = 95

        # The TLV Types
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_PRODUCT_NAME      = 0x01
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_PART_NUMBER       = 0x02
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_SERIAL_NUMBER     = 0x03
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_MAC_BASE          = 0x04
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_MANUF_DATE = 0x05
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_CARD_TYPE  = 0x06
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_HW_VERSION = 0x07
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_LABEL_REVISION    = 0x08
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_MODEL_NAME = 0x09
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_SW_VERSION = 0x0A
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_CRC        = 0x00
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_QUANTA_MAGIC      = 0xFF

        # Disable types not used by Quanta
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_MANUF_DATE        = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_DEVICE_VERSION    = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_PLATFORM_NAME     = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_ONIE_VERSION      = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_MAC_SIZE          = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_MANUF_NAME        = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_MANUF_COUNTRY     = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_VENDOR_NAME       = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_VENDOR_EXT        = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED
        eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_CRC_32            = \
            eeprom_tlvinfo.TlvInfoDecoder._TLV_CODE_UNDEFINED

    def switchaddrrange(self, e):
        # using 65 => (1 + 48 + 4 * 4)
        return 65

    def encode_checksum(self, crc):
        return binascii.unhexlify("%04x" % crc)

    def calculate_checksum(self, e):
        crc = super(board, self).calculate_checksum(e)
        calc_crc = ((crc & 0x0000ff00) >> 8  | (crc & 0x000000ff) << 8)
        return calc_crc

    def is_checksum_valid(self, e):
        '''
        Validate the checksum in the provided TlvInfo EEPROM data.
        QCT uses a 16bit crc and needs to be at the end of the used blocks
        '''
        tlv_index = 0
        tlv_end = eeprom_tlvinfo.TlvInfoDecoder._TLV_INFO_MAX_LEN

        while (tlv_index + 2) < len(e) and tlv_index < tlv_end:
            if not self.is_valid_tlv(e[tlv_index:]):
                print "Invalid TLV field starting at EEPROM offset %d" % (tlv_index,)
                return
            if ord(e[tlv_index]) == self._TLV_CODE_QUANTA_CRC:
                tlv_crc = ord(e[tlv_index+2]) <<  8 | ord(e[tlv_index+3])
                calc_crc = self.calculate_checksum(e[:tlv_index+2]) # include the CRC TLV header
                if tlv_crc == calc_crc:
                    return(True, calc_crc)
                else:
                    return (False, calc_crc)

            tlv_index += ord(e[tlv_index+1]) + 2

        return (False, calc_crc)


