#!/bin/sh

#
# Copyright 2015,2015,2018 Cumulus Networks, Inc.  All rights reserved
# All rights reserved.
#

# Invoked by /etc/init.d/hw_init

# Purpose #1
#
# Determine the fan configuration for multiple Quanta LY2 SKUs and
# setup the fan related configuration files accordingly.
#
# Front-to-back air flow systems use fans 1 through 4.  These fans
# have a maximum RPM of 13000.
#
# Back-to-front air flow systems use fans 5 through 8.  These fans
# have a maximum RPM of 22000.
#
# The CY8C3245 fan controller has knobs for the 8 separate fans.
# 
# First we try reading the EEPROM.  The 'part number' node should tell
# us what type of system we have.  Failing that we read all 8 fan
# tachometers to determine which type of system we have.  Unconnected
# fans return a zero tachometer reading.
#
# The fan configuration files need only be setup once after a fresh
# install.

# Purpose #2
#
# Based on the the air flow the fan controller needs a little tweaking
# in order to adjust the slew rate of the fan sppeds.
#
# This must be done every time the system boots.
#
# The power on default of the CY8C3245 controller is for the 13000 RPM
# front-to-back systems.  When the 22000 RPM back-to-front system is
# detected we need to tweak the controller for the faster settings.

# Cache previously detected fan configuration
CACHE_DIR=/run/cache/cumulus/fan_config
CACHE_CONF="$CACHE_DIR/state"

# The following config file depend on the fan devices
SENSORS_CONF=/etc/sensors.d/quanta.conf
FAN_MONITOR_CONF=/etc/fan_monitor/fan_monitor.cfg
PWMD_CONF=/etc/pwmd.conf

[ "$1" = "start" ] || exit 0

# Load the VERBOSE setting and other rcS variables
. /lib/init/vars.sh

# Define LSB log_* functions.
# Depend on lsb-base (>= 3.2-14) to ensure that this file is present
# and status_of_proc is working.
. /lib/lsb/init-functions

detect_fans()
{
    # First try the EEPROM
    # Part Number: 1LY2BZZ000R -- F2B
    # Part Number: 1LY2BZZ000S -- B2F
    part_num=$(decode-syseeprom |grep 'Part Number'|awk '{print $5}')
    if [ "$part_num" = "1LY2BZZ000R" ] ; then
        FAN_CONFIG="F2B"
    elif [ "$part_num" = "1LY2BZZ000S" ] ; then
        FAN_CONFIG="B2F"
    else
        # Detect what fans we do have

        # Read fan values, keeping a count of how many fan RPMs are greater
        # than zero.
        F2B_CNT=0
        B2F_CNT=0
        IDX=1
        while [ $IDX -le 8 ] ; do
            fan_input="$CY8C3245_SYSFS/fan${IDX}_input"
            [ -r $fan_input ] || {
                log_failure_msg "Unable to find CY8C3245 fan${IDX}: $fan_input"
                exit 1
            }
            val=$(cat $fan_input)
            if [ $val -gt 0 ] ; then
                if [ $IDX -lt 5 ] ; then
                    # Fans 1-4 are front-to-back
                    F2B_CNT=$(( $F2B_CNT + 1 ))
                else
                    # Fans 5-8 are back-to-front
                    B2F_CNT=$(( $B2F_CNT + 1 ))
                fi
            fi
            IDX=$(( $IDX + 1 ))
        done

        if [ $F2B_CNT -eq 0 ] && [ $B2F_CNT -eq 0 ] ; then
            log_failure_msg "All CY8C3245 fans read back zero RPM"
            exit 1
        fi

        # Use whichever count is greater.  If there is a tie then go with
        # front-to-back.
        if [ $F2B_CNT -ge $B2F_CNT ] ; then
            FAN_CONFIG="F2B"
        else
            FAN_CONFIG="B2F"
        fi
    fi

    # Store result in the cache
    mkdir -p $CACHE_DIR || {
        log_failure_msg "Problems making cache dir: $CACHE_DIR"
    }
    echo $FAN_CONFIG > $CACHE_CONF || {
        log_failure_msg "Problems writing cache file: $CACHE_CONF"
    }

}

# Construct common config file fragments
PLATFORM="Quanta LY2"
MIN_RPM=2500
MAX_F2B_RPM=14000
MAX_B2F_RPM=23000

# make_sensors_conf() -- Based on the fan type generate an appropriate
# sensors configuration file.
make_sensors_conf()
{
    config=$1

    # Emit common sensors config file text
    mkdir -p $(dirname $SENSORS_CONF)
    (cat <<EOF
# libsensors configuration file for $PLATFORM

chip "linux_bde_tmon-*"

     set temp1_max 100
     set temp1_max_hyst 100

chip "CY8C3245-*"

     set temp1_max 68
     set temp2_max 68
     set temp3_max 68
     set temp4_max 68
     set temp5_max 68

     # The units of fanN_max and fanN_min are revolutions per minute
     # (RPM).

EOF
    ) > $SENSORS_CONF

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
        (cat <<EOF
     set fan1_max $MAX_RPM
     set fan1_min $MIN_RPM
     set fan2_max $MAX_RPM
     set fan2_min $MIN_RPM
     set fan3_max $MAX_RPM
     set fan3_min $MIN_RPM
     set fan4_max $MAX_RPM
     set fan4_min $MIN_RPM

     # For systems with front-to-back air flow fans 5,6,7 and 8 are
     # unconnected.

     ignore fan5
     ignore fan6
     ignore fan7
     ignore fan8

EOF
        ) >> $SENSORS_CONF
    else
        # Use back-to-front configuration
        (cat <<EOF
     set fan5_max $MAX_RPM
     set fan5_min $MIN_RPM
     set fan6_max $MAX_RPM
     set fan6_min $MIN_RPM
     set fan7_max $MAX_RPM
     set fan7_min $MIN_RPM
     set fan8_max $MAX_RPM
     set fan8_min $MIN_RPM

     # For systems with back-to-front air flow fans 1,2,3 and 4 are
     # unconnected.

     ignore fan1
     ignore fan2
     ignore fan3
     ignore fan4

EOF
        ) >> $SENSORS_CONF
    fi

}

# make_pwmd_conf() -- Based on the fan type generate an appropriate
# pwmd configuration file.
make_pwmd_conf()
{
    config=$1

    # Emit common pwmd config file text
    mkdir -p $(dirname $PWMD_CONF)
    (cat <<EOF
# pwmd configuration for $PLATFORM

[DEFAULT]
interval = 20

# This platform sets the target RPM instead of PWM.
rpmmode = 1

# For this platform minpwm/maxpwm are in units of RPM
minpwm = $MIN_RPM
maxpwm = $MAX_RPM

# Temperature Sensors
monitor = linux_bde_tmon-pci-0100-temp1
          CY8C3245-i2c-4-2e-temp1
          CY8C3245-i2c-4-2e-temp2
          CY8C3245-i2c-4-2e-temp3
          CY8C3245-i2c-4-2e-temp4 
          CY8C3245-i2c-4-2e-temp5

# Fan specific configuration sections

EOF
    ) > $PWMD_CONF

    if [ $config = "F2B" ] ; then
        # Use front-to-back configuration
        (cat <<EOF
[CY8C3245-i2c-4-2e-pwm1]

[CY8C3245-i2c-4-2e-pwm2]

[CY8C3245-i2c-4-2e-pwm3]

[CY8C3245-i2c-4-2e-pwm4]

EOF
        ) >> $PWMD_CONF
    else
        # Use back-to-front configuration
        (cat <<EOF
[CY8C3245-i2c-4-2e-pwm5]

[CY8C3245-i2c-4-2e-pwm6]

[CY8C3245-i2c-4-2e-pwm7]

[CY8C3245-i2c-4-2e-pwm8]

EOF
        ) >> $PWMD_CONF
    fi

}

# make_fan_monitor_conf() -- Based on the fan type generate an appropriate
# fan_monitor configuration file.
make_fan_monitor_conf()
{
    config=$1

    # Emit common fan_monitor config file text
    mkdir -p $(dirname $FAN_MONITOR_CONF)
    (cat <<EOF
# Fan monitor configuration for $PLATFORM

[Control]

num_fans = 4
sensor_type = CY8C3245-i2c-4

[Fans]

fan1_high = $MAX_RPM

fan2_high = $MAX_RPM

fan3_high = $MAX_RPM

fan4_high = $MAX_RPM

EOF
    ) > $FAN_MONITOR_CONF

}

# Main program starts here

CY8C3245_SYSFS=/sys/bus/i2c/devices/4-002e

# Sanity checking:
[ -d $CY8C3245_SYSFS ] || {
    log_failure_msg "Unable to find CY8C3245 sensor chip: $CY8C3245_SYSFS"
    exit 1
}

if [ -r $CACHE_CONF ] ; then
    FAN_CONFIG=$(cat $CACHE_CONF)
    if [ $FAN_CONFIG != "F2B" ] && [ $FAN_CONFIG != "B2F" ] ; then
        log_failure_msg "Unexpected fan config: $FAN_CONFIG"
        exit 1
    fi
else
    detect_fans
fi

if [ $FAN_CONFIG = "F2B" ] ; then
    MAX_RPM=$MAX_F2B_RPM
else
    MAX_RPM=$MAX_B2F_RPM
fi

# The configuration files need only be created once.  If they already
# exist do not touch them.

# sensors config
[ -r $SENSORS_CONF ] || make_sensors_conf $FAN_CONFIG

# pwmd config
[ -r $PWMD_CONF ] || make_pwmd_conf $FAN_CONFIG

# fan monitor config
[ -r $FAN_MONITOR_CONF ] || make_fan_monitor_conf $FAN_CONFIG

# Adjust fan controller slew rate. This must be done every boot.
#
# *_DUTY ranges from 0 (0% duty cycle) to 10000 (100% duty cycle)
# *_RPM ranges from 0 to the maximum RPM

if [ $FAN_CONFIG = "F2B" ] ; then
    # Use the numbers gleaned from the power on defaults
    echo  5000 > $CY8C3245_SYSFS/fan_low_duty
    echo  6800 > $CY8C3245_SYSFS/fan_low_rpm
    echo 10000 > $CY8C3245_SYSFS/fan_high_duty
    echo 13000 > $CY8C3245_SYSFS/fan_high_rpm
    echo  3000 > $CY8C3245_SYSFS/fan_speed_0_duty
    echo 13000 > $CY8C3245_SYSFS/fan_speed_100_duty
else
    # Use empirically derived numbers that gave good response
    # characteristics.
    echo  2000 > $CY8C3245_SYSFS/fan_low_duty
    echo  4400 > $CY8C3245_SYSFS/fan_low_rpm
    echo  8000 > $CY8C3245_SYSFS/fan_high_duty
    echo 17600 > $CY8C3245_SYSFS/fan_high_rpm
    echo  3000 > $CY8C3245_SYSFS/fan_speed_0_duty
    echo 22000 > $CY8C3245_SYSFS/fan_speed_100_duty
fi
